@@ -2,6 +2,9 @@ from lightwood.ensemble.base import BaseEnsemble
 from lightwood.ensemble.best_of import BestOf
 from lightwood.ensemble.mean_ensemble import MeanEnsemble
 from lightwood.ensemble.mode_ensemble import ModeEnsemble
+from lightwood.ensemble.stacked_ensemble import StackedEnsemble
+from lightwood.ensemble.ts_stacked_ensemble import TsStackedEnsemble
 from lightwood.ensemble.weighted_mean_ensemble import WeightedMeanEnsemble
 
-__all__ = ['BaseEnsemble', 'BestOf', 'MeanEnsemble', 'ModeEnsemble', 'WeightedMeanEnsemble']
+__all__ = ['BaseEnsemble', 'BestOf', 'MeanEnsemble', 'ModeEnsemble', 'WeightedMeanEnsemble', 'StackedEnsemble',
+           'TsStackedEnsemble']
