@@ -5,62 +5,6 @@ import numpy as np
 from lightwood.api.dtype import dtype
 
 
-'''
-def is_allowed(v):
-    if v is None:
-        return True
-
-    if isinstance(v, bool):
-        return True
-
-    try:
-        float(v)
-        return True
-    except:
-        pass
-
-    if v in ['True', 'False']:
-        return True
-
-    if isinstance(v, str):
-        if v.startswith('"') and v.endswith('"'):
-            return True
-        if v.startswith("'") and v.endswith("'"):
-            return True
-
-    # Predictor member
-    if v.startswith('self.') and '(' not in v and len(v) < 50:
-        return True
-
-    # Allowed variable names
-    if v in ['df', 'data', 'encoded_data', 'train_data', 'encoded_train_data', 'test_data']:
-        return True
-
-    try:
-        cv = dict(v)
-        for k in cv:
-            ka = is_allowed(k)
-            ma = is_allowed(cv[k])
-            if not ka or not ma:
-                return False
-        return True
-    except Exception:
-        pass
-
-    try:
-        cv = list(v)
-        for ma in cv:
-            ma = is_allowed(m)
-            if not ma:
-                return False
-        return True
-    except Exception:
-        pass
-
-    raise Exception(f'Possible code injection: {v}')
-'''
-
-
 def is_allowed(v):
     if '(' in str(v):
         return False
